@@ -2,7 +2,7 @@
 //
 // Index.cpp: Indexing Objects
 //
-// Copyright (C) 2016-2024    Xiuwen Zheng
+// Copyright (C) 2016-2025    Xiuwen Zheng
 //
 // This file is part of SeqArray.
 //
@@ -21,6 +21,7 @@
 
 #include <cstdio>
 #include "Index.h"
+#include <sys/stat.h>
 
 using namespace std;
 
@@ -933,6 +934,14 @@ vector<C_Int32> &CFileInfo::Position()
 	return _Position;
 }
 
+void CFileInfo::ClearPosition()
+{
+	if (!_Root)
+		throw ErrSeqArray(ERR_FILE_ROOT);
+	_Position.clear();
+	std::vector<C_Int32>().swap(_Position);
+}
+
 CGenoIndex &CFileInfo::GenoIndex()
 {
 	if (_GenoIndex.Empty())
@@ -1093,13 +1102,14 @@ bool CVarApplyList::CallNext()
 
 static const int PROGRESS_BAR_CHAR_NUM = 50;
 static const int PROGRESS_LINE_NUM = 10000;
+static const int PROGRESS_INTERVAL_SECOND = 10;
 
 static const double S_MIN  =  60;
 static const double S_HOUR =  60 * S_MIN;
 static const double S_DAY  =  24 * S_HOUR;
 static const double S_YEAR = 365 * S_DAY;
 
-static const char *time_str(double s)
+COREARRAY_DLL_LOCAL const char *time_str(double s)
 {
 	if (R_FINITE(s))
 	{
@@ -1121,27 +1131,26 @@ static const char *time_str(double s)
 
 static const char *datetime_str()
 {
-	static char date_buffer[96];
+	static char buffer[96];
 	time_t rawtime;
 	time(&rawtime);
 	struct tm *p = localtime(&rawtime);
-	snprintf(date_buffer, sizeof(date_buffer), "%04d-%02d-%02d %02d:%02d:%02d",
+	snprintf(buffer, sizeof(buffer), "%04d-%02d-%02d %02d:%02d:%02d",
 		p->tm_year+1900, p->tm_mon+1, p->tm_mday,
 		p->tm_hour, p->tm_min, p->tm_sec);
-	return date_buffer;
+	return buffer;
 }
 
 
-CProgress::CProgress(C_Int64 start, C_Int64 count, SEXP conn, bool newline)
+CProgress::CProgress(C_Int64 count, SEXP Rconn, bool verbose)
 {
 	vTotalCount = count;
-	vCounter = (start >= 0) ? start : 0;
+	vCounter = 0;
 	double percent;
-	File = NULL;
-	if (conn && !Rf_isNull(conn))
-		File = R_GetConnection(conn);
-	FwdCnt = 0;
-	NewLine = newline;
+	OutFile = NULL;
+	if (Rconn && !Rf_isNull(Rconn))
+		OutFile = R_GetConnection(Rconn);
+	Verbose = verbose;
 
 	if (count > 0)
 	{
@@ -1159,19 +1168,17 @@ CProgress::CProgress(C_Int64 start, C_Int64 count, SEXP conn, bool newline)
 	}
 
 	time_t s; time(&s);
-	_start_time = s;
+	_start_time = _last_time = s;
 	_timer.reserve(128);
 	_timer.push_back(pair<double, time_t>(percent, s));
 
 	ShowProgress();
 }
 
-CProgress::~CProgress()
-{ }
+CProgress::~CProgress() { }
 
 void CProgress::Forward(C_Int64 Inc)
 {
-	FwdCnt++;
 	vCounter += Inc;
 	if (vTotalCount > 0 && vCounter > vTotalCount)
 		vCounter = vTotalCount;
@@ -1179,173 +1186,140 @@ void CProgress::Forward(C_Int64 Inc)
 	{
 		if (vTotalCount > 0)
 		{
-			while (vCounter >= _hit)
+			while (_hit <= vCounter)
 			{
 				_start += _step;
 				_hit = (C_Int64)(_start);
 			}
 			if (_hit > vTotalCount) _hit = vTotalCount;
 		} else {
-			while (vCounter >= _hit) _hit += PROGRESS_LINE_NUM;
+			while (_hit <= vCounter) _hit += PROGRESS_LINE_NUM;
 		}
 		ShowProgress();
 	}
 }
 
+static bool file_exists(const string &fname)
+{
+	struct stat sb;
+	return (stat(fname.c_str(), &sb) == 0);
+}
+
+static bool if_show_idx_cnt(int &idx, int &cnt)
+{
+	if (R_Block_Count > 0)
+	{
+		idx = R_Block_Index;
+		cnt = R_Block_Count;
+	} else if (R_Process_Count && R_Process_Index && (*R_Process_Count>0))
+	{
+		idx = *R_Process_Index;
+		cnt = *R_Process_Count;
+	} else
+		return false;
+	return true;
+}
+
 void CProgress::ShowProgress()
 {
-	if (File)
-	{
-		if (vTotalCount > 0)
-		{
-			char bar[PROGRESS_BAR_CHAR_NUM + 1];
-			double p = (double)vCounter / vTotalCount;
-			int n = (int)round(p * PROGRESS_BAR_CHAR_NUM);
-			memset(bar, '.', sizeof(bar));
-			memset(bar, '=', n);
-			if ((vCounter > 0) && (n < PROGRESS_BAR_CHAR_NUM))
-				bar[n] = '>';
-			bar[PROGRESS_BAR_CHAR_NUM] = 0;
-
-			// ETC: estimated time to complete
-			n = (int)_timer.size() - 20;  // 20% as a sliding window size
-			if (n < 0) n = 0;
-			time_t now; time(&now);
-			_timer.push_back(pair<double, time_t>(p, now));
-
-			// in seconds
-			double s = difftime(now, _timer[n].second);
-			double diff = p - _timer[n].first;
-			if (diff > 0)
-				s = s / diff * (1 - p);
-			else
-				s = R_NaN;
-			p *= 100;
-			// if completed
-			if (vCounter >= vTotalCount) s = difftime(now, _start_time);
-
-			// show
-			if (NewLine)
-			{
-				ConnPutText(File, "[%s] %2.0f%%, %s %s", bar, p,
-					vCounter < vTotalCount ? "ETC:" : "completed,", time_str(s));
-				if (R_Process_Count && R_Process_Index && (*R_Process_Count>1))
-					ConnPutText(File, " (process %d/%d)", *R_Process_Index, *R_Process_Count);
-				ConnPutText(File, "\n");
-			} else {
-				ConnPutText(File, "\r[%s] %2.0f%%, %s %s", bar, p,
-					vCounter < vTotalCount ? "ETC:" : "completed,", time_str(s));
-				if (R_Process_Count && R_Process_Index && (*R_Process_Count>1))
-					ConnPutText(File, " (process %d/%d)", *R_Process_Index, *R_Process_Count);
-				ConnPutText(File, "    ");
-				if (vCounter >= vTotalCount) ConnPutText(File, "\n");
-			}
-		} else {
-			int n = vCounter / PROGRESS_LINE_NUM;
-			n = (n / 100) + (n % 100 ? 1 : 0);
-			string s(n, '.');
-			const char *dt = datetime_str();
-			if (NewLine)
-			{
-				if (vCounter > 0)
-				{
-					ConnPutText(File, "[:%s (%dk lines)] %s", s.c_str(),
-						vCounter/1000, dt);
-				} else
-					ConnPutText(File, "[: (0 line)] %s", dt);
-				if (R_Process_Count && R_Process_Index && (*R_Process_Count>1))
-					ConnPutText(File, " (process %d/%d)", *R_Process_Index, *R_Process_Count);
-				ConnPutText(File, "\n");
-			} else {
-				if (vCounter > 0)
-				{
-					ConnPutText(File, "\r[:%s (%dk lines)] %s", s.c_str(),
-						vCounter/1000, dt);
-				} else
-					ConnPutText(File, "\r[: (0 line)] %s", dt);
-				if (R_Process_Count && R_Process_Index && (*R_Process_Count>1))
-					ConnPutText(File, " (process %d/%d)", *R_Process_Index, *R_Process_Count);
-			}
-		}
-		(*File->fflush)(File);
-	}
-}
-
-
-CProgressStdOut::CProgressStdOut(C_Int64 count, int nproc, bool verbose):
-	CProgress(0, count, NULL, false)
-{
-	if (count < 0)
-		throw ErrSeqArray("%s, 'count' should be greater than zero.", __func__);
-	_last_time = _timer.back().second;
-	NProcess = (nproc > 0) ? nproc : 1;
-	Verbose = verbose;
-	ShowProgress();
-}
-
-void CProgressStdOut::ShowProgress()
-{
-	if (Verbose && (vTotalCount > 0))
+	if (!OutFile && !Verbose) return;  // no output
+	if (vTotalCount > 0)
 	{
+		// progress bar to be displayed
 		char bar[PROGRESS_BAR_CHAR_NUM + 1];
+		memset(bar, '.', PROGRESS_BAR_CHAR_NUM);
+		bar[PROGRESS_BAR_CHAR_NUM] = 0;
 		double p = (double)vCounter / vTotalCount;
 		int n = (int)round(p * PROGRESS_BAR_CHAR_NUM);
-		memset(bar, '.', sizeof(bar));
 		memset(bar, '=', n);
-		if ((vCounter > 0) && (n < PROGRESS_BAR_CHAR_NUM))
-			bar[n] = '>';
-		bar[PROGRESS_BAR_CHAR_NUM] = 0;
+		if ((vCounter > 0) && (n < PROGRESS_BAR_CHAR_NUM)) bar[n] = '>';
 
 		// ETC: estimated time to complete
-		n = (int)_timer.size() - 20;  // 20% as a sliding window size
+		n = (int)_timer.size() - 20;  // ~20% as a sliding window size
 		if (n < 0) n = 0;
 		time_t now; time(&now);
 		_timer.push_back(pair<double, time_t>(p, now));
-
 		// in seconds
-		double interval = difftime(now, _last_time);
 		double s = difftime(now, _timer[n].second);
 		double diff = p - _timer[n].first;
 		if (diff > 0)
-		{
 			s = s / diff * (1 - p);
-			if ((NProcess > 1) && (FwdCnt > 0) && (FwdCnt <= NProcess))
-			{
-				// correction based on beta distribution beta(FwdCnt, NProcess+1-FwdCnt)
-				s = s / (NProcess + 1) * FwdCnt * 2;
-			}			
-		} else
+		else
 			s = R_NaN;
 		p *= 100;
+		// if completed
+		if (vCounter >= vTotalCount) s = difftime(now, _start_time);
 
-		// show
-		_last_time = now;
-		if (vCounter >= vTotalCount)
+		// output to a file / R connection
+		if (OutFile)
 		{
-			char buffer[512];
-			s = difftime(_last_time, _start_time);
-			int n = snprintf(buffer, sizeof(buffer),
-				"\r[%s] 100%%, completed, %s", bar, time_str(s));
-			if (R_Process_Count && R_Process_Index && (*R_Process_Count>1))
-			{
-				snprintf(buffer+n, sizeof(buffer)-n, " (process %d/%d)",
-					*R_Process_Index, *R_Process_Count);
-			}
-			Rprintf("%s\n", buffer);
-		} else if ((interval >= 5) || (vCounter <= 0))
-		{
-			char buffer[512];
-			_last_time = now;
-			int n = snprintf(buffer, sizeof(buffer),
-				"\r[%s] %2.0f%%, ETC: %s", bar, p, time_str(s));
-			if ((vCounter>0) && R_Process_Count && R_Process_Index && (*R_Process_Count>1))
-			{
-				n += snprintf(buffer+n, sizeof(buffer)-n, " (process %d/%d)",
-					*R_Process_Index, *R_Process_Count);
-			}
-			strcpy(buffer+n, "    ");
-			Rprintf("%s", buffer);
+			ConnPutText(OutFile, "[%s] %2.0f%%, %s %s", bar, p,
+				vCounter < vTotalCount ? "ETC:" : "used", time_str(s));
+			int i, m;
+			if (if_show_idx_cnt(i, m))
+				ConnPutText(OutFile, " (%d/%d)", i, m);
+			ConnPutText(OutFile, "\n");
+			(*OutFile->fflush)(OutFile);
 		}
+
+		// output to stdout
+		if (Verbose)
+		{
+			// check whether display based on the status files
+			if (R_Process_Count && (*R_Process_Count > 1) &&
+				R_Process_Index && (*R_Process_Index > 1) &&
+				(vCounter < vTotalCount))
+			{
+				if (R_Process_StatusFName.size() < *R_Process_Count) return;
+				const int idx = (*R_Process_Index) - 1;
+				for (int i=0; i < idx; i++)
+					if (file_exists(R_Process_StatusFName[i])) return;
+			}
+			// display
+			char buf[512];
+			if (vCounter >= vTotalCount)
+			{
+				int n = snprintf(buf, sizeof(buf),
+					"\r[%s] %2.0f%%, used %s", bar, p, time_str(s));
+				int i, m;
+				if (if_show_idx_cnt(i, m))
+					snprintf(buf+n, sizeof(buf)-n, " (%d/%d)", i, m);
+				if (R_Process_Count && *R_Process_Count>0)
+					Rprintf("%s    ", buf);
+				else
+					Rprintf("%s    \n", buf);
+			} else {
+				double interval = difftime(now, _last_time);  // in seconds
+				if ((vCounter <= 0) || (interval >= PROGRESS_INTERVAL_SECOND))
+				{
+					_last_time = now;
+					int n = snprintf(buf, sizeof(buf),
+						"\r[%s] %2.0f%%, ETC: %s", bar, p, time_str(s));
+					int i, m;
+					if (if_show_idx_cnt(i, m))
+						n += snprintf(buf+n, sizeof(buf)-n, " (%d/%d)", i, m);
+					Rprintf("%s    ", buf);
+				}
+			}
+		}
+
+	} else if (OutFile)
+	{
+		int n = vCounter / PROGRESS_LINE_NUM;
+		n = (n / 100) + (n % 100 ? 1 : 0);
+		string s(n, '.');
+		const char *dt = datetime_str();
+		if (vCounter > 0)
+		{
+			ConnPutText(OutFile, "[:%s (%dk lines)] %s", s.c_str(),
+				vCounter/1000, dt);
+		} else
+			ConnPutText(OutFile, "[: (0 line)] %s", dt);
+		int i, m;
+		if (if_show_idx_cnt(i, m))
+			ConnPutText(OutFile, " (%d/%d)", i, m);
+		ConnPutText(OutFile, "\n");
+		(*OutFile->fflush)(OutFile);
 	}
 }
 
@@ -1362,8 +1336,11 @@ SEXP R_Data_Name      = R_NilValue;
 SEXP R_Data_Dim2_Name = R_NilValue;
 SEXP R_Data_ListClass = R_NilValue;
 
-int* R_Process_Count = NULL;
-int* R_Process_Index = NULL;
+int* R_Process_Index = NULL;  // the index of child process
+int* R_Process_Count = NULL;  // the number of child processes
+vector<string> R_Process_StatusFName;  // list of file names for child processes
+int R_Block_Index = 0;  // the index of block being used currently
+int R_Block_Count = 0;  // the number of blocks, 0 when it is not used
 
 
 
